class CorpusAnalysisApp {
  constructor(corpusId) {
    this.data = {};

    // HTML elements
    this.elements = {
      container: document.querySelector('#corpus-analysis-app-container'),
      extensionTabs: document.querySelector('#corpus-analysis-app-extension-tabs'),
      initModal: document.querySelector('#corpus-analysis-app-init-modal'),
      overview: document.querySelector('#corpus-analysis-app-overview')
    };
    // Materialize elements
    this.elements.m = {
      extensionTabs: M.Tabs.init(this.elements.extensionTabs),
      initModal: M.Modal.init(this.elements.initModal, {dismissible: false})
    };

    this.extensions = {};

    this.settings = {
      corpusId: corpusId
    };
  }

  init() {
    this.disableActionElements();
    this.elements.m.initModal.open();
    // Init data
    this.data.cQiClient = new CQiClient(this.settings.corpusId);
    this.data.cQiClient.connect()
      .then(cQiStatus => {
        return this.data.cQiClient.corpora.get(`NOPAQUE_${this.settings.corpusId}`);
      })
      .then(
        cQiCorpus => {
          this.data.corpus = {o: cQiCorpus};
          this.data.corpus.o.getVisualizationData()
            .then(
              (data) => {
                console.log(data);
                this.renderGeneralCorpusInfo(data);
                this.renderTextInfoList(data);
                this.renderTextProportionsGraphic(data);
                this.renderWordFrequenciesGraphic(data);
              }
            );
          this.data.corpus.o.getCorpusData()
            .then(corpusData => {
              // this.renderGeneralCorpusInfo(corpusData);
              // this.renderTextInfoList(corpusData);
              // this.renderTextProportionsGraphic(corpusData);
              // this.renderWordFrequenciesGraphic(corpusData);
              this.renderWordDistributionsGraphic(corpusData);
            });
          // TODO: Don't do this hgere
          cQiCorpus.updateDb();
          this.enableActionElements();
          for (let extension of Object.values(this.extensions)) {extension.init();}
          this.elements.m.initModal.close();
        },
        cQiError => {
          let errorsElement = this.elements.initModal.querySelector('.errors');
          let progressElement = this.elements.initModal.querySelector('.progress');
          errorsElement.innerText = JSON.stringify(cQiError);
          errorsElement.classList.remove('hide');
          progressElement.classList.add('hide');
          if ('payload' in cQiError && 'code' in cQiError.payload && 'msg' in cQiError.payload) {
            app.flash(`${cQiError.payload.code}: ${cQiError.payload.msg}`, 'error');
          }
        }
      );

  
    // Add event listeners
    for (let extensionSelectorElement of this.elements.overview.querySelectorAll('.extension-selector')) {
      extensionSelectorElement.addEventListener('click', () => {
        this.elements.m.extensionTabs.select(extensionSelectorElement.dataset.target);
      });
    }
  }

  registerExtension(extension) {
    if (extension.name in this.extensions) {
      console.error(`Can't register extension ${extension.name}: Already registered`);
      return;
    }
    this.extensions[extension.name] = extension;
    if ('cQiClient' in this.data && this.data.cQiClient.connected) {extension.init();}
  }

  disableActionElements() {
    let actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (let actionElement of actionElements) {
      if (actionElement.nodeName === 'INPUT') {
        actionElement.disabled = true;
      } else if (actionElement.nodeName === 'SELECT') {
        actionElement.parentNode.querySelector('input.select-dropdown').disabled = true;
      } else {
        actionElement.classList.add('disabled');
      }
    }
  }

  enableActionElements() {
    let actionElements = this.elements.container.querySelectorAll('.corpus-analysis-action');
    for (let actionElement of actionElements) {
      if (actionElement.nodeName === 'INPUT') {
        actionElement.disabled = false;
      } else if (actionElement.nodeName === 'SELECT') {
        actionElement.parentNode.querySelector('input.select-dropdown').disabled = false;
      } else {
        actionElement.classList.remove('disabled');
      }
    }
  }

  renderGeneralCorpusInfo(corpusData) {
    document.querySelector('.corpus-num-tokens').innerHTML = corpusData.corpus.lexicon[0].counts.token;
    document.querySelector('.corpus-num-s').innerHTML = corpusData.corpus.lexicon[0].counts.s;
    document.querySelector('.corpus-num-unique-words').innerHTML = Object.entries(corpusData.corpus.lexicon[0].freqs.word).length;
    document.querySelector('.corpus-num-unique-lemmas').innerHTML = Object.entries(corpusData.corpus.lexicon[0].freqs.lemma).length;
    document.querySelector('.corpus-num-unique-pos').innerHTML = Object.entries(corpusData.corpus.lexicon[0].freqs.pos).length;
    document.querySelector('.corpus-num-unique-simple-pos').innerHTML = Object.entries(corpusData.corpus.lexicon[0].freqs.simple_pos).length;
  }

  renderTextInfoList(corpusData) {
    let corpusTextInfoListElement = document.querySelector('.corpus-text-info-list');
    let corpusTextInfoList = new CorpusTextInfoList(corpusTextInfoListElement);
    let texts = corpusData.text.lexicon;
    let textData = [];
    for (let i = 0; i < Object.entries(texts).length; i++) {
      let resource = {
        title: corpusData.lookups.text[i].title,
        publishing_year: corpusData.lookups.text[i].publishing_year,
        num_tokens: corpusData.text.lexicon[i].counts.token,
        num_sentences: corpusData.text.lexicon[i].counts.s,
        num_unique_words: Object.entries(corpusData.text.lexicon[i].freqs.word).length,
        num_unique_lemmas: Object.entries(corpusData.text.lexicon[i].freqs.lemma).length,
        num_unique_pos: Object.entries(corpusData.text.lexicon[i].freqs.pos).length,
        num_unique_simple_pos: Object.entries(corpusData.text.lexicon[i].freqs.simple_pos).length
      };
  
      textData.push(resource);
    }
    corpusTextInfoList.add(textData);

    let textCountChipElement = document.querySelector('.text-count-chip');
    textCountChipElement.innerHTML = `Text count: ${Object.values(corpusData.text.lexicon).length}`;
  }

  renderTextProportionsGraphic(corpusData) {
    let textProportionsGraphicElement = document.querySelector('#text-proportions-graphic');
    let texts = Object.entries(corpusData.text.lexicon);
    let graphData = [
      {
        values: texts.map(text => text[1].counts.token),
        labels: texts.map(text => `${corpusData.lookups.text[text[0]].title} (${corpusData.lookups.text[text[0]].publishing_year})`),
        type: 'pie'
      }
    ];
    let graphLayout = {
      height: 600,
      width: 600
    };
    Plotly.newPlot(textProportionsGraphicElement, graphData, graphLayout);
  }

  renderWordFrequenciesGraphic(corpusData) {
    let wordFrequenciesGraphicElement = document.querySelector('#word-frequencies-graphic');
    let words = Object.entries(corpusData.corpus.lexicon[0].freqs.word).sort((a, b) => b[1] - a[1]).slice(0, 5);
    let texts = Object.entries(corpusData.text.lexicon);
    let graphData = [];
    for (let word of words) {
      let data = {
        x: texts.map(text => `${corpusData.lookups.text[text[0]].title} (${corpusData.lookups.text[text[0]].publishing_year})`),
        y: texts.map(text => text[1].freqs.word[word[0]]),
        name: corpusData.lookups.word[word[0]],
        type: 'bar'
      };
      graphData.push(data);
    }

    let graphLayout = {
      height: 600,
      width: 600,
      barmode: 'stack',
      type: 'bar'
    };
    Plotly.newPlot(wordFrequenciesGraphicElement, graphData, graphLayout);
  }

  renderWordDistributionsGraphic(corpusData) {
    // let wordDistributionGraphicElement = document.querySelector('#word-distributions-graphic');
    // var trace1 = {
    //   x: [1, 2, 3, 4],
    //   y: [10, 11, 12, 13],
    //   mode: 'markers',
    //   marker: {
    //     size: [40, 60, 80, 100]
    //   }
    // };
    // var data = [trace1];
    // var layout = {
    //   title: 'Marker Size',
    //   showlegend: false,
    //   height: 500,
    //   width: 600
    // };
    // Plotly.newPlot(wordDistributionGraphicElement, data, layout);
  }
}
